@@ -37,7 +37,11 @@ export default {
     protocol: "协议",
     latency: "时延",
     lastSeenTime: "上次存活时间",
-    lastTryTime: "上次测试时间"
+    lastTryTime: "上次测试时间",
+    messages: {
+      notAllowInsecure:
+        "根据 {name} 的官方文档，如果你使用 {name}，将不允许 AllowInsecure"
+    }
   },
   subscription: {
     host: "域名",
@@ -96,7 +100,8 @@ export default {
     autoUpdateSub: "自动更新订阅",
     autoUpdateGfwlist: "自动更新GFWList",
     preferModeWhenUpdate: "解析订阅链接/更新时优先使用",
-    ipForwardOn: "开启局域网共享",
+    ipForwardOn: "开启IP转发",
+    portSharingOn: "开启端口分享",
     concurrency: "最大并发数",
     options: {
       global: "代理所有流量",
