@@ -52,7 +52,7 @@ module.exports = {
   // publicPath:process.env.NODE_ENV === 'production'
   // ? '/v2rayA/'
   // : '/',
-  outputDir: "../web",
+  outputDir: process.env.OUTPUT_DIR || "../web",
 
   // pwa: {
   //   name: "v2rayA",
