@@ -1,6 +1,7 @@
 package ntp
 
 import (
+	"fmt"
 	"github.com/beevik/ntp"
 	"time"
 )
@@ -12,7 +13,7 @@ const (
 func IsDatetimeSynced() (bool, time.Time, error) {
 	t, err := ntp.Time("ntp.aliyun.com")
 	if err != nil {
-		return false, time.Time{}, newError().Base(err)
+		return false, time.Time{}, fmt.Errorf("IsDatetimeSynced: %w", err)
 	}
 	if seconds := t.Sub(time.Now().UTC()).Seconds(); seconds >= 90 || seconds <= -90 {
 		return false, t, nil
