@@ -1,6 +1,7 @@
 package serverObj
 
 import (
+	"encoding/base64"
 	"fmt"
 	"github.com/v2rayA/v2rayA/common"
 	"github.com/v2rayA/v2rayA/core/coreObj"
@@ -117,6 +118,9 @@ func (s *Shadowsocks) Configuration(info PriorInfo) (c Configuration, err error)
 		Method:   s.Cipher,
 		Password: s.Password,
 	}
+	udpSupport := false
+	var proxySettings *coreObj.ProxySettings
+	var extraOutbounds []coreObj.OutboundObject
 	var chain []string
 	switch s.Plugin.Name {
 	case "simple-obfs":
@@ -133,7 +137,7 @@ func (s *Shadowsocks) Configuration(info PriorInfo) (c Configuration, err error)
 			RawQuery: url.Values{
 				"obfs": []string{s.Plugin.Opts.Obfs},
 				"host": []string{s.Plugin.Opts.Host},
-				"uri":  []string{s.Plugin.Opts.Uri},
+				"uri":  []string{s.Plugin.Opts.Path},
 			}.Encode(),
 		}
 		chain = append(chain, tcp.String(), simpleObfs.String())
@@ -142,10 +146,55 @@ func (s *Shadowsocks) Configuration(info PriorInfo) (c Configuration, err error)
 		default:
 			return c, fmt.Errorf("unsupported obfs %v of plugin %v", s.Plugin.Opts.Obfs, s.Plugin.Name)
 		}
-	// TODO: v2ray-plugin
-	//case "v2ray-plugin":
+	case "v2ray-plugin":
+		dialerTag := info.Tag + "-dialer"
+		proxySettings = &coreObj.ProxySettings{
+			Tag: dialerTag,
+		}
+		streamSettings := &coreObj.StreamSettings{}
+		host := s.Plugin.Opts.Host
+		if host == "" {
+			host = "cloudflare.com"
+		}
+		path := s.Plugin.Opts.Path
+		if !strings.HasPrefix(path, "/") {
+			path = "/" + path
+		}
+		if s.Plugin.Opts.Tls == "tls" {
+			streamSettings.Security = "tls"
+			streamSettings.TLSSettings = &coreObj.TLSSettings{}
+			// SNI
+			streamSettings.TLSSettings.ServerName = host
+		}
+		switch s.Plugin.Opts.Obfs {
+		case "quic":
+			return c, fmt.Errorf("quic is not yet supported")
+		default:
+			// "websocket" or ""
+			streamSettings.Network = "ws"
+			streamSettings.WsSettings = &coreObj.WsSettings{
+				Path: path,
+				Headers: coreObj.Headers{
+					Host: host,
+				},
+			}
+		}
+		extraOutbounds = append(extraOutbounds, coreObj.OutboundObject{
+			Tag:      dialerTag,
+			Protocol: "freedom",
+			Settings: coreObj.Settings{
+				DomainStrategy: "AsIs",
+				Redirect:       net.JoinHostPort(s.Server, strconv.Itoa(s.Port)),
+			},
+			StreamSettings: streamSettings,
+			Mux: &coreObj.Mux{
+				Enabled:     true,
+				Concurrency: 1,
+			},
+		})
 	case "":
 		// no plugin
+		udpSupport = true
 	default:
 		return c, fmt.Errorf("unsupported plugin %v", s.Plugin.Name)
 	}
@@ -156,9 +205,11 @@ func (s *Shadowsocks) Configuration(info PriorInfo) (c Configuration, err error)
 			Settings: coreObj.Settings{
 				Servers: []coreObj.Server{v2rayServer},
 			},
+			ProxySettings: proxySettings,
 		},
-		PluginChain: strings.Join(chain, ","),
-		UDPSupport:  true,
+		ExtraOutbounds: extraOutbounds,
+		PluginChain:    strings.Join(chain, ","),
+		UDPSupport:     udpSupport,
 	}, nil
 }
 
@@ -166,7 +217,7 @@ func (s *Shadowsocks) ExportToURL() string {
 	// sip002
 	u := &url.URL{
 		Scheme:   "ss",
-		User:     url.UserPassword(s.Cipher, s.Password),
+		User:     url.User(strings.TrimSuffix(base64.URLEncoding.EncodeToString([]byte(s.Cipher+":"+s.Password)), "=")),
 		Host:     net.JoinHostPort(s.Server, strconv.Itoa(s.Port)),
 		Fragment: s.Name,
 	}
@@ -187,10 +238,14 @@ func (s *Shadowsocks) GetProtocol() string {
 }
 
 func (s *Shadowsocks) ProtoToShow() string {
-	if s.Plugin.Name != "" {
-		return fmt.Sprintf("SS(%v+%v)", s.Cipher, s.Plugin.Opts.Obfs)
+	ciph := s.Cipher
+	if ciph == "chacha20-ietf-poly1305" || ciph == "chacha20-poly1305" {
+		ciph = "c20p1305"
 	}
-	return fmt.Sprintf("SS(%v)", s.Cipher)
+	if s.Plugin.Name != "" {
+		return fmt.Sprintf("SS(%v+%v)", ciph, s.Plugin.Name)
+	}
+	return fmt.Sprintf("SS(%v)", ciph)
 }
 
 func (s *Shadowsocks) GetHostname() string {
@@ -214,9 +269,10 @@ type Sip003 struct {
 	Opts Sip003Opts `json:"opts"`
 }
 type Sip003Opts struct {
+	Tls  string `json:"tls"`
 	Obfs string `json:"obfs"`
 	Host string `json:"host"`
-	Uri  string `json:"uri"`
+	Path string `json:"uri"`
 }
 
 func ParseSip003Opts(opts string) Sip003Opts {
@@ -224,15 +280,21 @@ func ParseSip003Opts(opts string) Sip003Opts {
 	fields := strings.Split(opts, ";")
 	for i := range fields {
 		a := strings.Split(fields[i], "=")
+		if len(a) == 1 {
+			// to avoid panic
+			a = append(a, "")
+		}
 		switch a[0] {
-		case "obfs":
+		case "tls":
+			sip003Opts.Tls = "tls"
+		case "obfs", "mode":
 			sip003Opts.Obfs = a[1]
-		case "obfs-path", "obfs-uri":
+		case "obfs-path", "obfs-uri", "path":
 			if !strings.HasPrefix(a[1], "/") {
 				a[1] += "/"
 			}
-			sip003Opts.Uri = a[1]
-		case "obfs-host":
+			sip003Opts.Path = a[1]
+		case "obfs-host", "host":
 			sip003Opts.Host = a[1]
 		}
 	}
@@ -241,10 +303,11 @@ func ParseSip003Opts(opts string) Sip003Opts {
 func ParseSip003(plugin string) Sip003 {
 	var sip003 Sip003
 	fields := strings.SplitN(plugin, ";", 2)
-	sip003.Name = fields[0]
-	switch sip003.Name {
+	switch fields[0] {
 	case "obfs-local", "simpleobfs":
 		sip003.Name = "simple-obfs"
+	default:
+		sip003.Name = fields[0]
 	}
 	sip003.Opts = ParseSip003Opts(fields[1])
 	return sip003
@@ -258,8 +321,8 @@ func (s *Sip003) String() string {
 	if s.Opts.Host != "" {
 		list = append(list, "obfs-host="+s.Opts.Host)
 	}
-	if s.Opts.Uri != "" {
-		list = append(list, "obfs-uri="+s.Opts.Uri)
+	if s.Opts.Path != "" {
+		list = append(list, "obfs-uri="+s.Opts.Path)
 	}
 	return strings.Join(list, ";")
 }
