@@ -80,9 +80,9 @@ func ParseTrojanURL(u string) (data *Trojan, err error) {
 
 func (t *Trojan) Configuration(info PriorInfo) (c Configuration, err error) {
 	if t.Protocol == "trojan-go" {
-		// "socks5:// -> tls,ws,ss,trojanc"
-		socks5 := url.URL{
-			Scheme: "socks5",
+		// "trojanc -> tcp:// -> ss,ws,tls"
+		tcpListener := url.URL{
+			Scheme: "tcp",
 			Host:   net.JoinHostPort("127.0.0.1", strconv.Itoa(info.PluginPort)),
 		}
 		tls := url.URL{
@@ -93,7 +93,7 @@ func (t *Trojan) Configuration(info PriorInfo) (c Configuration, err error) {
 				"allowInsecure": []string{common.BoolToString(t.AllowInsecure)},
 			}.Encode(),
 		}
-		chain := []string{socks5.String(), tls.String()}
+		chain := []string{tcpListener.String(), tls.String()}
 		if t.Type == "ws" {
 			ws := url.URL{
 				Scheme: "ws",
@@ -114,16 +114,20 @@ func (t *Trojan) Configuration(info PriorInfo) (c Configuration, err error) {
 			}
 			chain = append(chain, ss.String())
 		}
-		trojanc := url.URL{
-			Scheme: "trojanc",
-			User:   url.User(t.Password),
-			Host:   net.JoinHostPort(t.Server, strconv.Itoa(t.Port)),
-		}
-		chain = append(chain, trojanc.String())
 		return Configuration{
-			CoreOutbound: info.PluginObj(),
+			CoreOutbound: coreObj.OutboundObject{
+				Tag:      info.Tag,
+				Protocol: "trojan",
+				Settings: coreObj.Settings{
+					Servers: []coreObj.Server{{
+						Address:  "127.0.0.1",
+						Port:     info.PluginPort,
+						Password: t.Password,
+					}},
+				},
+			},
 			PluginChain:  strings.Join(chain, ","),
-			UDPSupport:   false,
+			UDPSupport:   true,
 		}, nil
 	}
 
