@@ -7,7 +7,8 @@ import (
 	"errors"
 	"fmt"
 	jsoniter "github.com/json-iterator/go"
-	"github.com/v2rayA/routingA"
+	"github.com/mohae/deepcopy"
+	"github.com/v2rayA/RoutingA"
 	"github.com/v2rayA/v2rayA/common"
 	"github.com/v2rayA/v2rayA/common/netTools/netstat"
 	"github.com/v2rayA/v2rayA/common/netTools/ports"
@@ -110,13 +111,16 @@ func parseDnsAddr(addr string) Addr {
 			}
 		}
 	}
-	// tcp://8.8.8.8:53, https://dns.google/dns-query
-	// TODO: quic:// uses UDP
+	// tcp://8.8.8.8:53, https://dns.google/dns-query, quic://dns.nextdns.io
 	if u, err := url.Parse(addr); err == nil {
+		udp := false
+		if u.Scheme == "quic" {
+			udp = true
+		}
 		return Addr{
 			host: u.Hostname(),
 			port: u.Port(),
-			udp:  false,
+			udp:  udp,
 		}
 	}
 	// dns.google, dns.pub, etc.
@@ -134,31 +138,56 @@ type DnsRouting struct {
 	ProxyIPs      []Addr
 }
 
-func appendAdvancedDnsServers(d *coreObj.DNS, lines []string, domains []string) {
+func parseAdvancedDnsServers(lines []string, domains []string) (domainNameServers []interface{}, routing []coreObj.RoutingRule) {
 	for _, line := range lines {
 		if len(line) == 0 {
 			continue
 		}
 		dns := ParseAdvancedDnsLine(line)
-		if u, err := url.Parse(dns.Val); err == nil && strings.Contains(dns.Val, "://") && !strings.Contains(u.Scheme, "://") {
+		if u, err := url.Parse(dns.Val); err == nil &&
+			strings.Contains(dns.Val, "://") &&
+			!strings.Contains(u.Scheme, "://") {
+			// FIXME: after quic:// supported
+			if u.Scheme == "quic" {
+				u.Scheme = "quic+local"
+				dns.Val = u.String()
+			}
 			if domains != nil {
-				d.Servers = append(d.Servers, coreObj.DnsServer{
+				domainNameServers = append(domainNameServers, coreObj.DnsServer{
 					Address: dns.Val,
 					Domains: domains,
 				})
 			} else {
-				d.Servers = append(d.Servers, dns.Val)
+				domainNameServers = append(domainNameServers, dns.Val)
 			}
 		} else {
 			addr := parseDnsAddr(dns.Val)
 			p, _ := strconv.Atoi(addr.port)
-			d.Servers = append(d.Servers, coreObj.DnsServer{
+			domainNameServers = append(domainNameServers, coreObj.DnsServer{
 				Address: addr.host,
 				Port:    p,
 				Domains: domains,
 			})
 		}
+
+		if dns.Val == "localhost" {
+			// no need to routing
+			continue
+		}
+		// we believe all lines are legal
+		var addr = parseDnsAddr(dns.Val)
+
+		if net.ParseIP(addr.host) != nil {
+			routing = append(routing, coreObj.RoutingRule{
+				Type: "field", InboundTag: []string{"dns"}, OutboundTag: dns.Out, IP: []string{addr.host}, Port: addr.port,
+			})
+		} else {
+			routing = append(routing, coreObj.RoutingRule{
+				Type: "field", InboundTag: []string{"dns"}, OutboundTag: dns.Out, Domain: []string{addr.host}, Port: addr.port,
+			})
+		}
 	}
+	return domainNameServers, routing
 }
 
 // outName -> isGroup
@@ -208,7 +237,7 @@ func (t *Template) FirstProxyOutboundName(filter func(outboundName string, isGro
 	return
 }
 
-func (t *Template) setDNS(outbounds []serverInfo, setting *configure.Setting, supportUDP map[string]bool) (routing []coreObj.RoutingRule, err error) {
+func (t *Template) setDNS(outbounds []serverInfo, supportUDP map[string]bool) (routing []coreObj.RoutingRule, err error) {
 	firstOutboundTag, _ := t.FirstProxyOutboundName(nil)
 	firstUDPSupportedOutboundTag, _ := t.FirstProxyOutboundName(func(outboundName string, isGroup bool) bool {
 		return supportUDP[outboundName]
@@ -216,7 +245,7 @@ func (t *Template) setDNS(outbounds []serverInfo, setting *configure.Setting, su
 	outboundTags := t.outNames()
 	var internal, external, all []string
 	var allThroughProxy = false
-	if setting.AntiPollution == configure.AntipollutionAdvanced {
+	if t.Setting.AntiPollution == configure.AntipollutionAdvanced {
 		// advanced
 		internal = configure.GetInternalDnsListNotNil()
 		external = configure.GetExternalDnsListNotNil()
@@ -249,10 +278,10 @@ func (t *Template) setDNS(outbounds []serverInfo, setting *configure.Setting, su
 				return nil, fmt.Errorf(`due to the protocol of outbound "%v" with no UDP supported, please use tcp:// and doh:// DNS rule instead, or change the connected server`, dns.Out)
 			}
 		}
-	} else if setting.AntiPollution != configure.AntipollutionClosed {
+	} else if t.Setting.AntiPollution != configure.AntipollutionClosed {
 		// preset
 		internal = []string{"223.6.6.6 -> direct", "119.29.29.29 -> direct"}
-		switch setting.AntiPollution {
+		switch t.Setting.AntiPollution {
 		case configure.AntipollutionAntiHijack:
 			break
 		case configure.AntipollutionDnsForward:
@@ -280,35 +309,21 @@ func (t *Template) setDNS(outbounds []serverInfo, setting *configure.Setting, su
 		// guess the user want to protect the privacy
 		t.DNS.DisableFallback = &True
 	}
-	if setting.AntiPollution != configure.AntipollutionClosed {
+	if t.Setting.AntiPollution != configure.AntipollutionClosed {
 		if len(external) == 0 {
 			// not split traffic
-			appendAdvancedDnsServers(t.DNS, internal, nil)
+			d, r := parseAdvancedDnsServers(internal, nil)
+			t.DNS.Servers = append(t.DNS.Servers, d...)
+			routing = append(routing, r...)
 		} else {
 			// split traffic
-			appendAdvancedDnsServers(t.DNS, external, nil)
-			appendAdvancedDnsServers(t.DNS, internal, []string{"geosite:cn"})
-		}
-	}
-	// routing
-	dnsList := append(append([]string{}, internal...), external...)
-	for _, line := range dnsList {
-		dns := ParseAdvancedDnsLine(line)
-		if dns.Val == "localhost" {
-			// no need to routing
-			continue
-		}
-		// we believe all lines are legal
-		var addr = parseDnsAddr(dns.Val)
+			d, r := parseAdvancedDnsServers(external, nil)
+			t.DNS.Servers = append(t.DNS.Servers, d...)
+			routing = append(routing, r...)
 
-		if net.ParseIP(addr.host) != nil {
-			routing = append(routing, coreObj.RoutingRule{
-				Type: "field", InboundTag: []string{"dns"}, OutboundTag: dns.Out, IP: []string{addr.host}, Port: addr.port,
-			})
-		} else {
-			routing = append(routing, coreObj.RoutingRule{
-				Type: "field", InboundTag: []string{"dns"}, OutboundTag: dns.Out, Domain: []string{addr.host}, Port: addr.port,
-			})
+			d, r = parseAdvancedDnsServers(internal, []string{"geosite:cn"})
+			t.DNS.Servers = append(t.DNS.Servers, d...)
+			routing = append(routing, r...)
 		}
 	}
 
@@ -350,54 +365,78 @@ func (t *Template) setDNS(outbounds []serverInfo, setting *configure.Setting, su
 	domainsToLookup = common.Deduplicate(domainsToLookup)
 	var domainsToHosts []string
 	if len(domainsToLookup) > 0 {
-		if service.CheckDohSupported() == nil {
-			t.DNS.Servers = append(t.DNS.Servers, coreObj.DnsServer{
-				Address:      "https://doh.pub/dns-query",
-				Domains:      domainsToLookup,
-				SkipFallback: true,
-			})
-			t.DNS.Servers = append(t.DNS.Servers, coreObj.DnsServer{
-				Address:      "https://doh.alidns.com/dns-query",
-				Domains:      domainsToLookup,
-				SkipFallback: true,
-			})
-			domainsToHosts = append(domainsToHosts, "doh.pub")
-			domainsToHosts = append(domainsToHosts, "doh.alidns.com")
+		var dnsList []string
+		if service.CheckTcpDnsSupported() == nil {
+			dnsList = []string{
+				"tcp://208.67.220.220:5353 -> direct",
+				"tcp://119.29.29.29:53 -> direct",
+			}
 		} else {
-			t.DNS.Servers = append(t.DNS.Servers, coreObj.DnsServer{
-				Address:      "dns.pub",
-				Domains:      domainsToLookup,
-				SkipFallback: true,
-			})
-			t.DNS.Servers = append(t.DNS.Servers, coreObj.DnsServer{
-				Address:      "dns.alidns.com",
-				Domains:      domainsToLookup,
-				SkipFallback: true,
-			})
-			domainsToHosts = append(domainsToHosts, "dns.pub")
-			domainsToHosts = append(domainsToHosts, "dns.alidns.com")
+			dnsList = []string{
+				"208.67.220.220:5353 -> direct",
+				"119.29.29.29:53 -> direct",
+			}
 		}
+		d, r := parseAdvancedDnsServers(dnsList, domainsToLookup)
+		t.DNS.Servers = append(t.DNS.Servers, d...)
+		routing = append(routing, r...)
 	}
 	// set hosts
+	var wg sync.WaitGroup
+	var mu sync.Mutex
+	ech := make(chan error, len(domainsToHosts))
 	for _, domain := range domainsToHosts {
-		ips, err := resolv.LookupHost(domain)
-		if err != nil {
-			return routing, fmt.Errorf("[Error] %w: please make sure you're connected to the Internet", err)
+		wg.Add(1)
+		go func(domain string) {
+			defer wg.Done()
+			ips, err := resolv.LookupHost(domain)
+			if err != nil {
+				ech <- fmt.Errorf("%w: please make sure you're connected to the Internet", err)
+				return
+			}
+			ips = FilterIPs(ips)
+			mu.Lock()
+			if t.DNS.Hosts == nil {
+				t.DNS.Hosts = make(coreObj.Hosts)
+			}
+			if service.CheckHostsListSupported() == nil {
+				t.DNS.Hosts[domain] = ips
+			} else {
+				t.DNS.Hosts[domain] = ips[0]
+			}
+			mu.Unlock()
+		}(domain)
+	}
+	wg.Wait()
+	select {
+	case err = <-ech:
+		return nil, err
+	default:
+		// deduplicate
+		strRouting := make([]string, 0, len(routing))
+		for _, r := range routing {
+			b, err := jsoniter.Marshal(r)
+			if err != nil {
+				log.Fatal("%v", err)
+			}
+			strRouting = append(strRouting, string(b))
 		}
-		if t.DNS.Hosts == nil {
-			t.DNS.Hosts = make(coreObj.Hosts)
+		strRouting = common.Deduplicate(strRouting)
+		routing = routing[:0]
+		for _, sr := range strRouting {
+			var r coreObj.RoutingRule
+			if err := jsoniter.Unmarshal([]byte(sr), &r); err != nil {
+				log.Fatal("%v: %v", err, sr)
+			}
+			routing = append(routing, r)
 		}
-		ips = FilterIPs(ips)
-		if service.CheckHostsListSupported() == nil {
-			t.DNS.Hosts[domain] = ips
-		} else {
-			t.DNS.Hosts[domain] = ips[0]
-		}
+		return routing, nil
 	}
-	return
 }
 
-// The order are from v4 IPs to v6 IPs. If the system does not support IPv6, v6 IPs will not be returned.
+// FilterIPs returns filtered IP list.
+// The order are from v4 IPs to v6 IPs.
+// If the system does not support IPv6, v6 IPs will not be returned.
 func FilterIPs(ips []string) []string {
 	var ret []string
 	for _, ip := range ips {
@@ -557,23 +596,37 @@ func (t *Template) AppendRoutingRuleByMode(mode configure.RulePortMode, inbounds
 			coreObj.RoutingRule{
 				Type:        "field",
 				OutboundTag: "direct",
-				InboundTag:  []string{"rule"},
+				InboundTag:  inbounds,
 			},
 		)
 	case configure.RoutingAMode:
-		if err := parseRoutingA(t, []string{"rule"}); err != nil {
+		if err := parseRoutingA(t, inbounds); err != nil {
 			return err
 		}
 	}
 	return nil
 }
 
-func (t *Template) setRulePortRouting(setting *configure.Setting) error {
-	return t.AppendRoutingRuleByMode(setting.RulePortMode, []string{"rule"})
+func (t *Template) setRulePortRouting() error {
+	return t.AppendRoutingRuleByMode(t.Setting.RulePortMode, []string{"rule"})
 }
 func parseRoutingA(t *Template, routingInboundTags []string) error {
-	ra := configure.GetRoutingA()
-	rules, err := routingA.Parse(ra)
+	lines := strings.Split(configure.GetRoutingA(), "\n")
+	hardcodeReplacement := regexp.MustCompile(`\$\$.+?\$\$`)
+	for i := range lines {
+		hardcodes := hardcodeReplacement.FindAllString(lines[i], -1)
+		for _, hardcode := range hardcodes {
+			env := strings.TrimSuffix(strings.TrimPrefix(hardcode, "$$"), "$$")
+			val, ok := os.LookupEnv(env)
+			if !ok {
+				log.Error("RoutingA: Environment variable \"%v\" is not found", env)
+			} else {
+				log.Info("RoutingA: Environment variable %v=%v", env, strconv.Quote(val))
+			}
+			lines[i] = strings.Replace(lines[i], hardcode, val, 1)
+		}
+	}
+	rules, err := RoutingA.Parse(strings.Join(lines, "\n"))
 	if err != nil {
 		log.Warn("parseRoutingA: %v", err)
 		return err
@@ -581,11 +634,11 @@ func parseRoutingA(t *Template, routingInboundTags []string) error {
 	defaultOutbound, _ := t.FirstProxyOutboundName(nil)
 	for _, rule := range rules {
 		switch rule := rule.(type) {
-		case routingA.Define:
+		case RoutingA.Define:
 			switch rule.Name {
 			case "inbound", "outbound":
 				switch o := rule.Value.(type) {
-				case routingA.Bound:
+				case RoutingA.Bound:
 					proto := o.Value
 					switch proto.Name {
 					case "http", "socks":
@@ -698,7 +751,7 @@ func parseRoutingA(t *Template, routingInboundTags []string) error {
 	outboundTags := t.outNames()
 	for _, rule := range rules {
 		switch rule := rule.(type) {
-		case routingA.Define:
+		case RoutingA.Define:
 			switch rule.Name {
 			case "default":
 				switch v := rule.Value.(type) {
@@ -709,12 +762,12 @@ func parseRoutingA(t *Template, routingInboundTags []string) error {
 					}
 				}
 			}
-		case routingA.Routing:
-			rr := coreObj.RoutingRule{
+		case RoutingA.Routing:
+			rr := deepcopy.Copy(coreObj.RoutingRule{
 				Type:        "field",
 				OutboundTag: rule.Out,
 				InboundTag:  routingInboundTags,
-			}
+			}).(coreObj.RoutingRule)
 			for _, f := range rule.And {
 				switch f.Name {
 				case "domain", "domains":
@@ -768,8 +821,8 @@ func parseRoutingA(t *Template, routingInboundTags []string) error {
 	return nil
 }
 
-func (t *Template) setTransparentRouting(setting *configure.Setting) (err error) {
-	switch setting.Transparent {
+func (t *Template) setTransparentRouting() (err error) {
+	switch t.Setting.Transparent {
 	case configure.TransparentProxy:
 	case configure.TransparentWhitelist:
 		return t.AppendRoutingRuleByMode(configure.WhitelistMode, []string{"transparent"})
@@ -812,7 +865,7 @@ func (t *Template) AppendDokodemo(tproxy *string, port int, tag string) {
 	t.Inbounds = append(t.Inbounds, dokodemo)
 }
 
-func (t *Template) setOutboundSockopt(setting *configure.Setting) {
+func (t *Template) SetOutboundSockopt() {
 	mark := 0x80
 	//tos := 184
 	for i := range t.Outbounds {
@@ -828,22 +881,21 @@ func (t *Template) setOutboundSockopt(setting *configure.Setting) {
 		if t.Outbounds[i].Protocol == "freedom" && t.Outbounds[i].Tag == "direct" {
 			t.Outbounds[i].Settings.DomainStrategy = "UseIP"
 		}
-		if setting.TcpFastOpen != configure.Default {
-			tmp := setting.TcpFastOpen == configure.Yes
+		if t.Setting.TcpFastOpen != configure.Default {
+			tmp := t.Setting.TcpFastOpen == configure.Yes
 			t.Outbounds[i].StreamSettings.Sockopt.TCPFastOpen = &tmp
 		}
 		t.checkAndSetMark(&t.Outbounds[i], mark)
 	}
 }
-func (t *Template) setDualStack(setting *configure.Setting) {
+func (t *Template) setDualStack() {
 	const (
 		tag4Suffix = "_ipv4"
 		tag6Suffix = "_ipv6"
 	)
 	tagMap := make(map[string]struct{})
-	inbounds6 := make([]coreObj.Inbound, len(t.Inbounds))
-	copy(inbounds6, t.Inbounds)
-	if !setting.IntranetSharing {
+	inbounds6 := deepcopy.Copy(t.Inbounds).([]coreObj.Inbound)
+	if !t.Setting.PortSharing {
 		// copy a group of ipv6 inbounds and set the tag
 		for i := range t.Inbounds {
 			if t.Inbounds[i].Tag == "dns-in" {
@@ -879,7 +931,7 @@ func (t *Template) setDualStack(setting *configure.Setting) {
 					tag6 = append(tag6, tag+tag6Suffix)
 				}
 			}
-			if len(tag6) > 0 && iptables.IsIPv6Supported() {
+			if v6supported := iptables.IsIPv6Supported(); len(tag6) > 0 && v6supported {
 				t.Routing.Rules[i].InboundTag = append(t.Routing.Rules[i].InboundTag, tag6...)
 			}
 		}
@@ -969,7 +1021,7 @@ func SetVlessGrpcInbound(vlessGrpc *coreObj.Inbound) (err error) {
 	return nil
 }
 
-func (t *Template) setInbound(setting *configure.Setting) error {
+func (t *Template) setInbound() error {
 	p := configure.GetPortsNotNil()
 	if p != nil {
 		t.Inbounds[0].Port = p.Socks5
@@ -992,11 +1044,11 @@ func (t *Template) setInbound(setting *configure.Setting) error {
 			t.Inbounds = append(t.Inbounds[:i], t.Inbounds[i+1:]...)
 		}
 	}
-	if setting.Transparent != configure.TransparentClose {
+	if t.Setting.Transparent != configure.TransparentClose {
 		var tproxy string
-		switch setting.TransparentType {
+		switch t.Setting.TransparentType {
 		case configure.TransparentTproxy, configure.TransparentRedirect:
-			tproxy = string(setting.TransparentType)
+			tproxy = string(t.Setting.TransparentType)
 		}
 		t.AppendDokodemo(&tproxy, 32345, "transparent")
 	}
@@ -1226,6 +1278,7 @@ func (t *Template) resolveOutbounds(
 	}
 	// keep order with serverInfos
 	outboundTags = make([]string, len(serverData.ServerInfos))
+	var extraOutbounds []coreObj.OutboundObject
 	var outbounds []_outbound
 	for obj, sInfos := range serverData.ServerObj2ServerInfos() {
 		var (
@@ -1265,6 +1318,7 @@ func (t *Template) resolveOutbounds(
 				if err != nil {
 					return nil, nil, err
 				}
+				extraOutbounds = append(extraOutbounds, c.ExtraOutbounds...)
 				var s plugin.Server
 				if len(c.PluginChain) > 0 {
 					s, err = plugin.ServerFromChain(c.PluginChain)
@@ -1293,6 +1347,7 @@ func (t *Template) resolveOutbounds(
 			if err != nil {
 				return nil, nil, err
 			}
+			extraOutbounds = append(extraOutbounds, c.ExtraOutbounds...)
 			for _, v := range balancers {
 				c.CoreOutbound.Balancers = append(c.CoreOutbound.Balancers, v.name)
 			}
@@ -1349,6 +1404,7 @@ func (t *Template) resolveOutbounds(
 		Tag:      "block",
 		Protocol: "blackhole",
 	})
+	t.Outbounds = append(t.Outbounds, extraOutbounds...)
 	return supportUDP, outboundTags, nil
 }
 
@@ -1455,11 +1511,11 @@ func NewTemplate(serverInfos []serverInfo, setting *configure.Setting) (t *Templ
 	t.OutboundTags = outboundTags
 
 	//set inbound ports according to the setting
-	if err = t.setInbound(setting); err != nil {
+	if err = t.setInbound(); err != nil {
 		return nil, err
 	}
 	//set DNS
-	dnsRouting, err := t.setDNS(serverInfos, setting, supportUDP)
+	dnsRouting, err := t.setDNS(serverInfos, supportUDP)
 	if err != nil {
 		return nil, err
 	}
@@ -1469,12 +1525,12 @@ func NewTemplate(serverInfos []serverInfo, setting *configure.Setting) (t *Templ
 	t.Routing.DomainMatcher = "mph"
 	t.setDNSRouting(dnsRouting, supportUDP)
 	//rule port routing
-	if err = t.setRulePortRouting(setting); err != nil {
+	if err = t.setRulePortRouting(); err != nil {
 		return nil, err
 	}
 	//transparent routing
-	if setting.Transparent != configure.TransparentClose {
-		if err = t.setTransparentRouting(setting); err != nil {
+	if t.Setting.Transparent != configure.TransparentClose {
+		if err = t.setTransparentRouting(); err != nil {
 			return nil, err
 		}
 	}
@@ -1508,13 +1564,13 @@ func NewTemplate(serverInfos []serverInfo, setting *configure.Setting) (t *Templ
 	t.optimizeGeoipMemoryOccupation()
 
 	//set outboundSockopt
-	t.setOutboundSockopt(setting)
+	t.SetOutboundSockopt()
 
 	//set fakedns destOverride
 	t.setInboundFakeDnsDestOverride()
 
 	//set inbound listening address and routing
-	t.setDualStack(setting)
+	t.setDualStack()
 
 	//check if there are any duplicated tags
 	if err = t.checkDuplicatedTags(); err != nil {
@@ -1591,13 +1647,12 @@ func PortOccupied(syntax []string) (err error) {
 			if p == nil {
 				continue
 			}
-			if ownPID := strconv.Itoa(os.Getpid());
-				p.PPID == ownPID ||
-					p.PID == ownPID {
+			if ownPID := strconv.Itoa(os.Getpid()); p.PPID == ownPID ||
+				p.PID == ownPID {
 				continue
 			}
 			occupiedErr := fmt.Errorf("%w by %v(%v): %v", OccupiedErr, p.Name, p.PID, s.LocalAddress.Port)
-			if configure.GetSettingNotNil().IntranetSharing {
+			if configure.GetSettingNotNil().PortSharing {
 				// want to listen 0.0.0.0, which conflicts with all IPs
 				return occupiedErr
 			}
@@ -1694,6 +1749,7 @@ func (t *Template) InsertMappingOutbound(o serverObj.ServerObj, inboundPort stri
 	var mark = 0x80
 	t.checkAndSetMark(&c.CoreOutbound, mark)
 	t.Outbounds = append(t.Outbounds, c.CoreOutbound)
+	t.Outbounds = append(t.Outbounds, c.ExtraOutbounds...)
 	iPort, err := strconv.Atoi(inboundPort)
 	if err != nil || iPort <= 0 {
 		return fmt.Errorf("port of inbound must be a positive number with string type")
