@@ -3,9 +3,10 @@ package configure
 import (
 	"fmt"
 	"github.com/v2rayA/v2rayA/common/resolv"
-	"log"
+	"github.com/v2rayA/v2rayA/pkg/util/log"
 	"net"
 	"sort"
+	"strconv"
 	"strings"
 	"time"
 )
@@ -172,14 +173,14 @@ func (w *Which) EqualTo(another Which) (ok bool) {
 }
 func (w *Which) Ping(timeout time.Duration) (err error) {
 	if w.TYPE == SubscriptionType {
-		return newError("you cannot ping a subscription")
+		return fmt.Errorf("you cannot ping a subscription")
 	}
 	tsr, err := w.LocateServerRaw()
 	if err != nil {
 		return
 	}
 	//BEGIN
-	host := tsr.VmessInfo.Add
+	host := tsr.ServerObj.GetHostname()
 	if net.ParseIP(host) == nil {
 		var hosts []string
 		hosts, err = resolv.LookupHost(host)
@@ -194,7 +195,7 @@ func (w *Which) Ping(timeout time.Duration) (err error) {
 		host = hosts[0]
 	}
 	t := time.Now()
-	conn, e := net.DialTimeout("tcp", net.JoinHostPort(host, tsr.VmessInfo.Port), timeout)
+	conn, e := net.DialTimeout("tcp", net.JoinHostPort(host, strconv.Itoa(tsr.ServerObj.GetPort())), timeout)
 	if e == nil || (strings.Contains(e.Error(), "refuse")) {
 		if e == nil {
 			_ = conn.Close()
@@ -202,50 +203,50 @@ func (w *Which) Ping(timeout time.Duration) (err error) {
 		//log.Println(host+":"+tsr.VmessInfo.Port, e)
 		w.Latency = fmt.Sprintf("%.0fms", time.Since(t).Seconds()*1000)
 	} else {
-		log.Println(e)
+		log.Debug("Ping: %v", e)
 		w.Latency = "TIMEOUT"
 	}
 	return
 }
 
-func (w *Which) LocateServerRaw() (sr *ServerRaw, err error) {
+func (w *Which) LocateServerRaw() (sr *ServerRawV2, err error) {
 	ind := w.ID - 1 //转化为下标
 	switch w.TYPE {
 	case ServerType:
-		servers := GetServers()
+		servers := GetServersV2()
 		if ind < 0 || ind >= len(servers) {
-			return nil, newError("LocateServerRaw: ID exceed range")
+			return nil, fmt.Errorf("LocateServerRaw: ID exceed range")
 		}
 		return &servers[ind], nil
 	case SubscriptionServerType:
-		subscriptions := GetSubscriptions()
+		subscriptions := GetSubscriptionsV2()
 		if w.Sub < 0 || w.Sub >= len(subscriptions) || ind < 0 || ind >= len(subscriptions[w.Sub].Servers) {
-			return nil, newError("LocateServerRaw: ID or Sub exceed range")
+			return nil, fmt.Errorf("LocateServerRaw: ID or Sub exceed range")
 		}
 		return &subscriptions[w.Sub].Servers[ind], nil
 	default:
-		return nil, newError("LocateServerRaw: invalid TYPE")
+		return nil, fmt.Errorf("LocateServerRaw: invalid TYPE")
 	}
 }
 
 func (ws *Whiches) FillLinks() (err error) {
-	servers := GetServers()
-	subscriptions := GetSubscriptions()
+	servers := GetServersV2()
+	subscriptions := GetSubscriptionsV2()
 	for _, w := range ws.Touches {
 		ind := w.ID - 1 //转化为下标
 		switch w.TYPE {
 		case ServerType:
 			if ind < 0 || ind >= len(servers) {
-				return newError("LocateServerRaw: ID exceed range")
+				return fmt.Errorf("LocateServerRaw: ID exceed range")
 			}
-			w.Link = servers[ind].VmessInfo.ExportToURL()
+			w.Link = servers[ind].ServerObj.ExportToURL()
 		case SubscriptionServerType:
 			if w.Sub < 0 || w.Sub >= len(subscriptions) || ind < 0 || ind >= len(subscriptions[w.Sub].Servers) {
-				return newError("LocateServerRaw: ID or Sub exceed range")
+				return fmt.Errorf("LocateServerRaw: ID or Sub exceed range")
 			}
-			w.Link = subscriptions[w.Sub].Servers[ind].VmessInfo.ExportToURL()
+			w.Link = subscriptions[w.Sub].Servers[ind].ServerObj.ExportToURL()
 		default:
-			return newError("LocateServerRaw: invalid TYPE")
+			return fmt.Errorf("LocateServerRaw: invalid TYPE")
 		}
 	}
 	return nil
@@ -284,7 +285,7 @@ func (ws *Whiches) SaveLatencies() (err error) {
 	}
 	// set subscriptions
 	for subIndex, serverIndexes := range subscriptionIndexes {
-		subRaw := GetSubscription(subIndex)
+		subRaw := GetSubscriptionV2(subIndex)
 		for index, which := range serverIndexes {
 			subRaw.Servers[index].Latency = which.Latency
 		}
