@@ -35,6 +35,10 @@ import (
 
 func checkEnvironment() {
 	config := conf.GetEnvironmentConfig()
+	if len(config.PrintReport) > 0 {
+		config.Report()
+		os.Exit(0)
+	}
 	if !config.PassCheckRoot || config.ResetPassword {
 		if os.Getegid() != 0 {
 			log.Fatal("Please execute this program with sudo or as a root user for the best experience.\n" +
@@ -272,6 +276,7 @@ func hello() {
 	wd, _ := os.Getwd()
 	log.Alert("v2rayA working directory is %v", wd)
 	log.Alert("v2rayA configuration directory is %v", conf.GetEnvironmentConfig().Config)
+	log.Alert("Golang: %v", runtime.Version())
 	log.Alert("OS: %v", runtime.GOOS)
 	log.Alert("Arch: %v", runtime.GOARCH)
 	log.Alert("Lite: %v", conf.GetEnvironmentConfig().Lite)
