@@ -7,22 +7,21 @@ import (
 	"github.com/gin-contrib/cors"
 	"github.com/gin-contrib/gzip"
 	"github.com/gin-gonic/gin"
-	"github.com/gookit/color"
 	"github.com/v2rayA/v2rayA/common"
+	"github.com/v2rayA/v2rayA/conf"
 	"github.com/v2rayA/v2rayA/db/configure"
-	"github.com/v2rayA/v2rayA/global"
+	"github.com/v2rayA/v2rayA/pkg/server/jwt"
+	"github.com/v2rayA/v2rayA/pkg/server/reqCache"
+	"github.com/v2rayA/v2rayA/pkg/util/log"
 	"github.com/v2rayA/v2rayA/server/controller"
-	"github.com/v2rayA/v2rayA/server/router/jwt"
-	"github.com/v2rayA/v2rayA/server/router/reqCache"
 	"io"
 	"io/fs"
-	"log"
 	"net"
 	"net/http"
 	"os"
 	"path"
 	"path/filepath"
-	"time"
+	"strings"
 )
 
 //go:embed web
@@ -38,11 +37,29 @@ func (c relativeFS) Open(name string) (fs.File, error) {
 	return c.root.Open(path.Join(c.relativeDir, name))
 }
 
+func cachedHTML(html []byte) func(ctx *gin.Context) {
+	etag := fmt.Sprintf("W/%x", md5.Sum(html))
+	h := string(html)
+	return func(ctx *gin.Context) {
+		if ctx.IsAborted() {
+			return
+		}
+		ctx.Header("Content-Type", "text/html; charset=utf-8")
+		ctx.Header("Cache-Control", "public, max-age=31536000")
+		ctx.Header("ETag", etag)
+		if match := ctx.GetHeader("If-None-Match"); match != "" {
+			if strings.Contains(match, etag) {
+				ctx.Status(http.StatusNotModified)
+				return
+			}
+		}
+		ctx.String(http.StatusOK, h)
+	}
+}
+
 func ServeGUI(engine *gin.Engine) {
-	data := []byte(time.Now().String())
-	etag := fmt.Sprintf("%x", md5.Sum(data))
 	r := engine.Use(gzip.Gzip(gzip.DefaultCompression))
-	webDir := global.GetEnvironmentConfig().WebDir
+	webDir := conf.GetEnvironmentConfig().WebDir
 	if webDir == "" {
 		webFS := relativeFS{
 			root:        webRoot,
@@ -64,26 +81,20 @@ func ServeGUI(engine *gin.Engine) {
 			})
 			return nil
 		})
-		r.GET("/", func(ctx *gin.Context) {
-			f, err := webFS.Open("index.html")
-			if err != nil {
-				ctx.Status(400)
-				return
-			}
-			defer f.Close()
-			b, err := io.ReadAll(f)
-			if err != nil {
-				ctx.Status(400)
-				return
-			}
-			ctx.Header("Cache-Control", "public, max-age=31536000")
-			ctx.Header("ETag", etag)
-			ctx.Header("Content-Type", "text/html; charset=utf-8")
-			ctx.String(http.StatusOK, string(b))
-		})
+
+		f, err := webFS.Open("index.html")
+		if err != nil {
+			log.Fatal("%v", err)
+		}
+		defer f.Close()
+		html, err := io.ReadAll(f)
+		if err != nil {
+			log.Fatal("%v", err)
+		}
+		r.GET("/", cachedHTML(html))
 	} else {
 		if _, err := os.Stat(webDir); os.IsNotExist(err) {
-			log.Printf("[Warning] web files cannot be found at %v. web UI cannot be served", webDir)
+			log.Warn("web files cannot be found at %v. web UI cannot be served", webDir)
 		} else {
 			filepath.Walk(webDir, func(path string, info os.FileInfo, err error) error {
 				if path == webDir {
@@ -96,16 +107,21 @@ func ServeGUI(engine *gin.Engine) {
 				r.StaticFile("/"+info.Name(), path)
 				return nil
 			})
-			engine.LoadHTMLFiles(path.Join(webDir, "index.html"))
-			r.GET("/", func(ctx *gin.Context) {
-				ctx.Header("Cache-Control", "public, max-age=31536000")
-				ctx.Header("ETag", etag)
-				ctx.HTML(http.StatusOK, "index.html", nil)
-			})
+
+			f, err := os.Open(path.Join(webDir, "index.html"))
+			if err != nil {
+				log.Fatal("%v", err)
+			}
+			defer f.Close()
+			html, err := io.ReadAll(f)
+			if err != nil {
+				log.Fatal("%v", err)
+			}
+			r.GET("/", cachedHTML(html))
 		}
 	}
 
-	app := global.GetEnvironmentConfig()
+	app := conf.GetEnvironmentConfig()
 
 	ip, port, _ := net.SplitHostPort(app.Address)
 	addrs, err := net.InterfaceAddrs()
@@ -182,9 +198,6 @@ func Run() error {
 		auth.GET("ports", controller.GetPorts)
 		auth.PUT("ports", controller.PutPorts)
 		//auth.PUT("account", controller.PutAccount)
-		auth.GET("portWhiteList", controller.GetPortWhiteList)
-		auth.PUT("portWhiteList", controller.PutPortWhiteList)
-		auth.POST("portWhiteList", controller.PostPortWhiteList)
 		auth.GET("dnsList", controller.GetDnsList)
 		auth.PUT("dnsList", controller.PutDnsList)
 		auth.GET("routingA", controller.GetRoutingA)
@@ -197,9 +210,9 @@ func Run() error {
 
 	ServeGUI(engine)
 
-	return engine.Run(global.GetEnvironmentConfig().Address)
+	return engine.Run(conf.GetEnvironmentConfig().Address)
 }
 
 func printRunningAt(address string) {
-	color.Red.Println("v2rayA is listening at", address)
+	log.Alert("v2rayA is listening at %v", address)
 }
